""" Functions for working with files in python """
from merlin import files


def test_read():
    assert 1 > 0


def test_readb():
    assert 1 > 0


def test_readlines():
    assert 1 > 0


def test_readlinesb():
    assert 1 > 0


def test_write():
    assert 1 > 0


def test_writeb():
    assert 1 > 0


def test_append():
    assert 1 > 0


def test_appendb():
    assert 1 > 0


def test_delete():
    assert 1 > 0


def test_exists():
    assert 1 > 0
